open Asttypes
open Longident
open Path
open Types
open Transltools
open Transltype
open Lambda
open Rdigest
open Misc

let debug = try ignore (Sys.getenv "DEBUG_BDIGEST"); true with _ -> false
let debug s = if debug then prerr_endline s else ()

let prim mdl name =
  try 
    transl_path (fst (Env.lookup_value (Ldot (Lident mdl, name)) Env.empty))
  with Not_found ->
    fatal_error ("Primitive "^mdl^"."^name ^ " not found.")

let transl_flags_rtype =
  transl_rtype (function
      Flag_Digest d -> Const_block (0, [Const_base (Const_string d)])
    | Flag_Regular x -> Const_block (1, [Const_base (Const_int x)])
    | Flag_Recursive x -> Const_block (2, [Const_base (Const_int x)]))

let transl_type_info_creation modname (id,decl) =
  try
    debug (Printf.sprintf "Bdigest: %s ...." (Ident.unique_name id));
    id,
    match decl.type_digest with
      DDigest d -> 
  	(* Info_Digest d *)
debug (Printf.sprintf "Bdigest: %s = %s" (Ident.unique_name id) d);
	make_type_info modname id
	  (Lconst (Const_block (0, [Const_base (Const_string d)])))
    | DNone -> 
debug (Printf.sprintf "Bdigest: %s = *ABSTRACT* (DNone)" (Ident.name id));
	(* Info_None *)
	make_type_info modname id
	  (Lconst (Const_pointer 0))
    | DLazy ->
debug (Printf.sprintf "Bdigest: %s = *ABSTRACT* (DLazy)" (Ident.name id));
	(* Info_None *)
	make_type_info modname id
	  (Lconst (Const_pointer 0))
    | DManifest rtyp -> 
debug (Printf.sprintf "Bdigest: %s = DManifest" (Ident.name id));
	make_type_info modname id
	  (Lconst (Const_block (1, [transl_rtype (fun d -> Const_base (Const_string d)) rtyp])))    
    | DSource src ->
debug (Printf.sprintf "Bdigest: %s = DSource" (Ident.name id));
	let get_flags_rtype = 
  	  map_rtype (fun x -> RTvar x) (fun ci typs ->
  	    match ci with
  	      Path p -> 
  		RTconstr (
  		  begin try
  		    Flag_Regular (list_mem_nth p src.regular)
  		  with
  		    _ ->
  		      match p with
  			Pident id ->
  			  begin try
  			    Flag_Recursive (list_mem_nth id src.recursive)
  			  with
  			    _ -> raise (Dtype.Generic_system_error ("transl_type_digest : non recursive type: " ^ Path.string_of p))
  			  end
  		      | _ -> raise (Dtype.Generic_system_error ("transl_type_digest : non regular type: " ^ Path.string_of p))
  		  end, typs)
  	    | Digest d -> RTconstr (Flag_Digest d, typs)) 
  	in
	match src.declarations with
	  [SType_manifest rtyp] ->
	    debug (Printf.sprintf "Bdigest: %s = Rdigest.manifest %s" 
	     (Ident.name id) 
	     (string_of_rtype (function
		 Digest d -> d
	       | Path p -> Path.string_of p) rtyp));
	    let frtyp = get_flags_rtype rtyp in
	    make_type_info modname id
  	      ( Lapply (prim "Rdigest" "manifest",
			[ Lconst (Const_base (Const_string (Ident.name id)));
			  Lconst (transl_flags_rtype frtyp);
  			  transl_list transl_path src.regular ]))
	| _ -> 
	    debug (Printf.sprintf "Bdigest: %s = Rdigest.digest ? [%s]" 
	     (Ident.name id) 
	     (catenate_sep "; " (List.map Path.string_of src.regular)) );
  	    let transl_source_declaration = function
  		SType_abstract -> Const_pointer 0
	      |	SType_none -> Const_pointer 1
  	      | SType_digest d -> Const_block (0, [Const_base (Const_string d)])
  	      | SType_manifest rtyp -> 
  		  Const_block (1, [transl_flags_rtype (get_flags_rtype rtyp)])
  	      | SType_variant typslist ->
  		  Const_block (2, [transl_const_list (transl_const_list 
			(fun typ -> transl_flags_rtype (get_flags_rtype typ))) 
			      typslist])
  	      | SType_record typs ->
  		  Const_block (3, [transl_const_list (fun typ -> 
		    transl_flags_rtype (get_flags_rtype typ)) typs])
  	    in
	    make_type_info modname id
	      (Lapply ( prim "Rdigest" "digest", 
		       [ Lconst (Const_base (Const_string (Ident.name id)));
			 Lconst (transl_const_list transl_source_declaration src.declarations);
			 transl_list transl_path src.regular ] ))
  with
    Dtype.Generic_system_error s -> raise (Dtype.Generic_system_error (Ident.name id ^ " : " ^ s))
