open Rdigest

(* external types *)

exception Type_match_failure of string rtype * string rtype * string * int * int

type lazy_info = lazy_type_info_cell ref

let rtype_to_etype rt = 
  let rec sub = function
      RTvar i -> RTvar i
    | RTarrow (f, t) -> RTarrow (sub f, sub t)
    | RTtuple tl -> RTtuple (List.map sub tl)
    | RTconstr ((rp,info), tl) -> 
	begin
	  let tl' = List.map sub tl in
	  match type_info info with 
	    Info_Digest d -> RTconstr (d, tl')
	  | Info_Manifest rt -> subst_rtype_at_var tl' rt 
	end
    | _ -> raise Exit
  in
  sub rt

let export_value_comp =
  fun rt oc v ->
    let et = rtype_to_etype rt in
    let ev = v, et in
    Marshal.to_channel oc ev []

let import_value_comp =
  fun (m,ls,le) rt ic ->
    let et = rtype_to_etype rt in
    let v, et' = Marshal.from_channel ic in (* Should check it is a tuple... *)
    if Dynamic.is_instance et' et then v
    else raise (Type_match_failure ( et, et', m, ls, le ))

let export_to_string_comp =
  fun rt v ->
    let et = rtype_to_etype rt in
    let ev = v, et in
    Marshal.to_string ev []

let import_from_string_comp =
  fun (m,ls,le) rt str ->
    let et = rtype_to_etype rt in
    let v, et' = Marshal.from_string str 0 in
    if Dynamic.is_instance et' et then v
    else raise (Type_match_failure ( et, et', m, ls, le ))

(*
let expand_digest_rtype (rtype : int rtype) (args : type_info list) = 
  let rec sub = function
      RTvar x -> RTvar x
    | RTarrow (f,t) -> RTarrow (sub f, sub t)
    | RTtuple typs -> RTtuple (List.map sub typs)
    | RTconstr (x, typs) ->
      begin
	match 
	  try
	    List.nth args x 
	  with _ -> 
	    raise (Failure ("Dynamic.expand_digest_rtype"^string_of_int x)) 
	with
	  Info_Digest d ->
	    RTconstr (d, List.map sub typs)
	| Info_Manifest typ ->
	    let subst_vars args typ =
	      let rec sub2 = function
		  RTvar x -> 
		    begin 
		      try List.nth args x 
		      with _ -> 
			raise (Failure "Dynamic.expand_digest_rtype: List.nth")
		    end
		| RTarrow (f,t) -> RTarrow (sub2 f, sub2 t)
		| RTtuple typs -> RTtuple (List.map sub2 typs)
		| RTconstr (d, typs) -> RTconstr (d, List.map sub2 typs)
	      in
	      sub2 typ
	    in
	    subst_vars (List.map sub typs) typ
      end
  in
  sub rtype
*)

