exception No_digest_info
exception Generic_rdigest_failure of string

val list_mem_nth : 'a -> 'a list -> int

val debug0 : string -> unit
val debug : string -> unit

(* built-in types *)
type rpath =
    RPident of string * int
  | RPdot of rpath * string * int
  | RPapply of rpath * rpath

type 'a rtype = 
    RTvar of int
  | RTarrow of 'a rtype * 'a rtype
  | RTtuple of 'a rtype list
  | RTconstr of 'a * 'a rtype list

type digest_info =
    Info_Digest of string (* = Digest.t *)
  | Info_Manifest of string rtype
  | Info_None

type lazy_digest_info_cell =
  | Value  of digest_info
  | Delayed of (unit -> digest_info)

type type_info = {
    path: rpath;
    mutable digest: lazy_digest_info_cell
  } 

val digest_info : type_info -> digest_info

type flags =
    Flag_Digest of string
  | Flag_Regular of int
  | Flag_Recursive of int

(* Type Digests *)

type 'a source_declaration =
    SType_abstract
  | SType_digest of string
  | SType_manifest of 'a rtype
  | SType_variant of 'a rtype list list
  | SType_record of 'a rtype list
  | SType_none

val catenate_sep : string -> string list -> string
val string_of_rpath : rpath -> string
val string_of_rtype : ('a -> string) -> 'a rtype -> string
val string_of_sdecl : ('a -> string) -> 'a source_declaration -> string
val iter_rtype :
  (int -> unit) ->
  ('a -> 'a rtype list -> unit) -> 'a rtype -> unit
val map_rtype :
  (int -> 'a rtype) ->
  ('b -> 'a rtype list -> 'a rtype) -> 'b rtype -> 'a rtype
val subst_rtype_at_var : 'a rtype list -> 'a rtype -> 'a rtype
val subst_rtype_at_flag_regular :
  digest_info list -> flags rtype -> string rtype
val subst_rtype_at_flag_regular2 :
  digest_info list -> flags rtype -> flags rtype
val subst_sdecl :
  digest_info list -> flags source_declaration -> flags source_declaration
val manifest : string -> flags rtype -> type_info list -> digest_info
val digest :
  string -> flags source_declaration list -> type_info list  -> digest_info
