open Dynamic

(* dynamic *)

let d = dyn 1;;

dyn (fun x -> x);;

let f x = dyn x;;

f (ref []);; (* This must be rejected !! *)

(* coerce -- single *)

(Dynamic.crc (dyn 1) : int);;

(Dynamic.crc (dyn []) : int list);; (* instantiation *)

(Dynamic.crc (dyn 1) : float);; (* runtime error *)

(* coerce -- multi *)

let print d =  coerce (dyn d) with
  (x : int) -> print_int x
| (x : float) -> print_float x
| (x : string) -> print_string x
;;

(* "any pattern" _ is available *)

let kind d =  coerce (dyn d) with
  (_ : int) -> print_string "int"
| (_ : float) -> print_string "float"
| (_ : string) -> print_string "string"
;;

(* if the type of a patten is clear, you need not to write an explicit
   type constraint *)

let kind d =  coerce (dyn d) with
  1 -> print_string "this is 1"
| (x : int) -> print_int x
;;

(* but, for type variables in the patterns, you have to write them *)

(* ERROR
let f d =  coerce (dyn d) with
  [x] -> print_int x
;;
*)

let f d =  coerce (dyn d) with
  [(x : int)] -> print_int x
;;

(* 'a list matches only with polymorphic list [] *)

let f d = coerce (dyn d) with
  (_ : 'a list) -> print_string "it is polymorphic"
| (_ : int list) -> print_string "it is int list"
;;

(* _ can also be used in type patterns. *)

let f d = coerce (dyn d) with
  (v, _ : int * _) -> print_string ("Left is int = " ^ string_of_int v)
| (_, v : _ * int) -> print_string ("Right is int = " ^ string_of_int v)
;;

(* default case *)

let f d = coerce (dyn d) with
  (v : int) -> print_int v
| _ -> raise (Failure "unsupported")
;;

(* the above one is equivalent with *)
let f d = coerce (dyn d) with
  (v : int) -> print_int v
| (_ : _) -> raise (Failure "unsupported")
;;

(* the polymorphism inside actions *)
(* but I am afraid it contains some bugs *)

let f d = coerce (dyn d) with
  (f : 'a -> 'a) -> f 1; f 1.0; ()
;;
