open Misc
open Longident
open Path
open Types

open Format

let print_layout layout = 
  List.iter (fun (p,o) ->
    print_string (Path.string_of p);
    print_string ":";
    print_int o;
    print_string " ")
    layout
;;

let comp_layout path cks =
  let rec sub_cases acc cks =
    List.flatten (List.map (fun ck ->
      List.flatten (List.map (sub_derived acc) ck.konst_derived)) cks)
  and sub_derived acc dk =
    match dk.derived_desc with
    | Some desc ->
	sub_konsts acc desc
    | None -> (* this is recursive, and must be already in acc *)
	acc
  and sub_konsts acc desc =
    match desc.val_konstraints with
    | Konst_None | Konst_Derived _ -> (* they are independent! *)
	[]
    | Konst_Cases (path,cks) ->
	if List.mem_assoc path acc then acc
	else sub_cases ((path, List.length cks) :: acc) cks
  in

  let path_len_lst = 
    let rec uniq l =
      List.rev (List.fold_right (fun p st -> 
	if List.mem p st then st else p :: st) l [])
    in
    let init = [path, List.length cks] in
    let lst = uniq (sub_cases init cks) in
    if lst = [] then init else lst
  in
  let offset = ref 0 in
  let layout =
    List.map (fun (p,len) ->
      let path_offset = p, !offset in
      offset := !offset + len;
      path_offset ) path_len_lst
  in
(*
  print_string "LAYOUT[";
  print_string (Path.string_of path);
  print_string " = ";
  print_layout layout;
  print_string "]";
*)
  layout
;;

let generic_compilation_layout konsts =
  match konsts with
  | Konst_Cases (path, cks) -> comp_layout path cks
  | _ -> []
;;

let recursives konsts =
  let rec recursives_konsts acc = function
    | Konst_None -> []
    | Konst_Cases (path, cks) ->
	if List.mem_assoc path acc then acc 
	else List.flatten (List.map (recursives_ck acc) cks)
    | Konst_Derived (path,dks) ->
	if List.mem_assoc path acc then acc 
	else List.flatten (List.map (recursives_dk acc) dks)
  and recursives_ck acc ck =
    List.flatten (List.map (recursives_dk acc) ck.konst_derived)
  and recursives_dk acc dk =
    match dk.derived_desc with
    | None -> (* recursive *) acc
    | Some vdesc -> 
	recursives_konsts ((dk.derived_path, vdesc)::acc) vdesc.val_konstraints
  in

  let rec uniq l =
    List.rev (List.fold_right (fun p st -> 
      if List.mem p st then st else p :: st) l [])
  in
  uniq (recursives_konsts [] konsts)
